// Copyright 2019 autocareai.com. All rights reserved.
// Use of this source code is governed by autocareai.com.

package task

import (
	"gadm-dws/apis"
	"gadm-dws/consts"
	"gadm-dws/model"
	"encoding/json"
	"github.com/tidwall/gjson"
	"gorm.io/gorm"
	"strings"
)

func handleInsertDws18(db *gorm.DB, modelNo, engineType string, dwsMessage *apis.DwsMessage, outputSourceCode string) (*apis.AdsMessage, error) {
	dws18 := &model.Dws18{}
	dws18.ModelNo = modelNo
	dws18.EngineType = engineType
	if dws18.ModelNo == "" || dws18.EngineType == "" {
		return nil, nil
	}

	err := dws18.Query(db, map[string]interface{}{"model_no": dws18.ModelNo, "engine_type": dws18.EngineType})
	if err != nil {
		if err != gorm.ErrRecordNotFound {
			return nil, err
		}
	} else {
		// 有数据
		return nil, nil
	}

	err = dws18.Insert(db)
	if err != nil {
		if !strings.Contains(err.Error(), "Duplicate") {
			return nil, err
		} else {
			return nil, nil
		}
	}

	adsMsg := NewAdsMessage(dwsMessage, outputSourceCode, consts.ACTIONINSERT)
	msgByte, _ := json.Marshal(*dws18)
	adsMsg.Content = string(msgByte)
	return adsMsg, nil
}

// vin模糊定型数据
func Dws18Task(db *gorm.DB, dwsMessage *apis.DwsMessage, outputSourceCode string) (adsMsgList []*apis.AdsMessage, dwsMsgList []*apis.DwsMessage, err error) {
	dws18 := &model.Dws18{}
	err = json.Unmarshal([]byte(dwsMessage.Content), dws18)
	if err != nil {
		return nil, nil, nil
	}

	if dws18.ModelNo == "" {
		return nil, nil, nil
	}

	/*err = dws18.Query(db, map[string]interface{}{"model_no": dws18.ModelNo, "engine_type": dws18.EngineType})
	if err != nil {
		if err != gorm.ErrRecordNotFound {
			return nil, nil, err
		}
	} else {
		// 有数据
		return nil, nil, nil
	}

	err = dws18.Insert(db)
	if err != nil {
		if !strings.Contains(err.Error(), "Duplicate") {
			return nil, nil, err
		} else {
			return nil, nil, nil
		}
	}

	adsMsg := NewAdsMessage(dwsMessage, outputSourceCode, consts.ACTIONINSERT)
	msgByte, _ := json.Marshal(*dws18)
	adsMsg.Content = string(msgByte)*/
	adsMsg, err := handleInsertDws18(db, dws18.ModelNo, dws18.EngineType, dwsMessage, outputSourceCode)
	if err != nil {
		return nil, nil, err
	}
	if adsMsg != nil {
		adsMsgList = append(adsMsgList, adsMsg)
	}
	if dws18.EngineInfo != "" {
		engineInfoList := gjson.Parse(dws18.EngineInfo).Array()
		for _, v := range engineInfoList {
			engineType := v.Get("engine_model").String()
			adsMsg, err := handleInsertDws18(db, dws18.ModelNo, engineType, dwsMessage, outputSourceCode)
			if err != nil {
				return nil, nil, err
			}
			if adsMsg != nil {
				adsMsgList = append(adsMsgList, adsMsg)
			}
		}
	}

	return adsMsgList, nil, nil
}
